#!/usr/bin/env python3

# adapted from http://www.rabbitmq.com/tutorials/tutorial-six-python.html

import pika
import uuid

class MarkdownRpcClient(object):
    def __init__(self):
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters(host='localhost')
        )

        self.channel = self.connection.channel()

        result = self.channel.queue_declare(queue='', exclusive=True)
        self.callback_queue = result.method.queue

        self.channel.basic_consume(on_message_callback=self.on_response, queue=self.callback_queue, auto_ack=True)

    def on_response(self, ch, method, props, body):
        if self.corr_id == props.correlation_id:
            self.response = body

    def call(self, markdown):
        self.response = None
        self.corr_id = str(uuid.uuid4())
        self.channel.basic_publish(
            exchange='', routing_key='rpc_queue',
            properties=pika.BasicProperties(
                reply_to = self.callback_queue,
                correlation_id = self.corr_id,
            ),
            body=markdown)

        while self.response is None:
            self.connection.process_data_events()

        return self.response.decode('utf8')

markdown_rpc = MarkdownRpcClient()
response = markdown_rpc.call("## Hello world\n\nGreetings.")
print(response)

